@@ -30,7 +30,7 @@ from phi.utils.log import logger
 from phi.workspace.template_to_repo_map import template_to_repo_map
 from phi.workspace.ws_enums import (
     WorkspaceEnv,
-    WorkspaceInfra,
+    WorkspaceConfigType,
     WorkspaceStarterTemplate,
     WorkspaceVisibilityEnum,
 )
@@ -536,7 +536,7 @@ def setup_workspace(ws_dir_path: Path) -> bool:
 def deploy_workspace(
     ws_data: PhiWsData,
     target_env: Optional[WorkspaceEnv] = None,
-    target_infra: Optional[WorkspaceInfra] = None,
+    target_config: Optional[WorkspaceConfigType] = None,
     target_name: Optional[str] = None,
     target_type: Optional[str] = None,
     dry_run: Optional[bool] = False,
@@ -550,24 +550,24 @@ def deploy_workspace(
 
     # Add configs in the order they should be deployed
     configs_to_deploy_infra_filter: List[InfraConfig] = []
-    if target_infra is None:
+    if target_config is None:
         if ws_config.docker is not None:
             configs_to_deploy_infra_filter.extend(ws_config.docker)
         if ws_config.aws is not None:
             configs_to_deploy_infra_filter.extend(ws_config.aws)
         if ws_config.k8s is not None:
             configs_to_deploy_infra_filter.extend(ws_config.k8s)
-    elif target_infra == WorkspaceInfra.docker:
+    elif target_config == WorkspaceConfigType.docker:
         if ws_config.docker is not None:
             configs_to_deploy_infra_filter.extend(ws_config.docker)
         else:
             print_error("No DockerConfig provided")
-    elif target_infra == WorkspaceInfra.k8s:
+    elif target_config == WorkspaceConfigType.k8s:
         if ws_config.k8s is not None:
             configs_to_deploy_infra_filter.extend(ws_config.k8s)
         else:
             print_error("No K8sConfig provided")
-    elif target_infra == WorkspaceInfra.aws:
+    elif target_config == WorkspaceConfigType.aws:
         if ws_config.aws is not None:
             configs_to_deploy_infra_filter.extend(ws_config.aws)
         else:
@@ -679,7 +679,7 @@ def deploy_workspace(
 def shutdown_workspace(
     ws_data: PhiWsData,
     target_env: Optional[WorkspaceEnv] = None,
-    target_infra: Optional[WorkspaceInfra] = None,
+    target_config: Optional[WorkspaceConfigType] = None,
     target_name: Optional[str] = None,
     target_type: Optional[str] = None,
     dry_run: Optional[bool] = False,
@@ -694,24 +694,24 @@ def shutdown_workspace(
 
     # Add configs in the order they should be shut down
     configs_to_shutdown_infra_filter: List[InfraConfig] = []
-    if target_infra is None:
+    if target_config is None:
         if ws_config.docker is not None:
             configs_to_shutdown_infra_filter.extend(ws_config.docker)
         if ws_config.k8s is not None:
             configs_to_shutdown_infra_filter.extend(ws_config.k8s)
         if ws_config.aws is not None:
             configs_to_shutdown_infra_filter.extend(ws_config.aws)
-    elif target_infra == WorkspaceInfra.docker:
+    elif target_config == WorkspaceConfigType.docker:
         if ws_config.docker is not None:
             configs_to_shutdown_infra_filter.extend(ws_config.docker)
         else:
             print_error("No DockerConfig provided")
-    elif target_infra == WorkspaceInfra.k8s:
+    elif target_config == WorkspaceConfigType.k8s:
         if ws_config.k8s is not None:
             configs_to_shutdown_infra_filter.extend(ws_config.k8s)
         else:
             print_error("No K8sConfig provided")
-    elif target_infra == WorkspaceInfra.aws:
+    elif target_config == WorkspaceConfigType.aws:
         if ws_config.aws is not None:
             configs_to_shutdown_infra_filter.extend(ws_config.aws)
         else:
@@ -818,6 +818,148 @@ def shutdown_workspace(
         print_subheading("Workspace shutdown failed")
 
 
+def patch_workspace(
+    ws_data: PhiWsData,
+    target_env: Optional[WorkspaceEnv] = None,
+    target_config: Optional[WorkspaceConfigType] = None,
+    target_name: Optional[str] = None,
+    target_type: Optional[str] = None,
+    dry_run: Optional[bool] = False,
+) -> None:
+    """Patch the Phidata Workspace. This is called from `phi ws patch`"""
+
+    if ws_data is None or ws_data.ws_config is None:
+        print_error("WorkspaceConfig invalid")
+        return
+
+    ws_config: WorkspaceConfig = ws_data.ws_config
+
+    # Add configs in the order they should be patched
+    configs_to_patch_infra_filter: List[InfraConfig] = []
+    if target_config is None:
+        if ws_config.docker is not None:
+            configs_to_patch_infra_filter.extend(ws_config.docker)
+        if ws_config.k8s is not None:
+            configs_to_patch_infra_filter.extend(ws_config.k8s)
+        if ws_config.aws is not None:
+            configs_to_patch_infra_filter.extend(ws_config.aws)
+    elif target_config == WorkspaceConfigType.docker:
+        if ws_config.docker is not None:
+            configs_to_patch_infra_filter.extend(ws_config.docker)
+        else:
+            print_error("No DockerConfig provided")
+    elif target_config == WorkspaceConfigType.k8s:
+        if ws_config.k8s is not None:
+            configs_to_patch_infra_filter.extend(ws_config.k8s)
+        else:
+            print_error("No K8sConfig provided")
+    elif target_config == WorkspaceConfigType.aws:
+        if ws_config.aws is not None:
+            configs_to_patch_infra_filter.extend(ws_config.aws)
+        else:
+            print_error("No AwsConfig provided")
+
+    configs_to_patch_env_filter: List[InfraConfig] = []
+    if target_env is None:
+        configs_to_patch_env_filter = configs_to_patch_infra_filter
+    else:
+        for config in configs_to_patch_infra_filter:
+            if config.env is None:
+                continue
+            if config.env.lower() not in WorkspaceEnv.values_list():
+                print_error(f"{config.env} not supported")
+                continue
+            config_env = WorkspaceEnv.from_str(config.env)
+
+            if config_env == target_env:
+                configs_to_patch_env_filter.append(config)
+    # logger.debug("Patching configs: {}".format(configs_to_shutdown_env_filter))
+
+    num_configs_to_process = len(configs_to_patch_env_filter)
+    num_configs_processed = 0
+    for config in configs_to_patch_env_filter:
+        if not isinstance(config, InfraConfig):
+            print_error(f"{config} is not an instance of InfraConfig")
+            continue
+        if not config.is_valid():
+            print_error("Config invalid")
+            continue
+        if not config.enabled:
+            logger.debug(
+                f"{config.__class__.__name__} for env {config.__class__.env} disabled"
+            )
+            num_configs_processed += 1
+            continue
+
+        ######################################################################
+        # NOTE: VERY IMPORTANT TO GET RIGHT
+        # Update sub-configs data using WorkspaceConfig
+        # 1. Pass down the paths from the WorkspaceConfig
+        #       These paths are used everywhere from Infra to Apps
+        # 2. Pass down common cloud configuration. eg: aws_region, aws_profile
+        ######################################################################
+
+        # The ws_dir_path is the ROOT directory for the workspace
+        config.workspace_root_path = ws_data.ws_dir_path
+        config.workspace_config_file_path = ws_data.ws_config_file_path
+        config.scripts_dir = ws_config.scripts_dir
+        config.storage_dir = ws_config.storage_dir
+        config.meta_dir = ws_config.meta_dir
+        config.products_dir = ws_config.products_dir
+        config.notebooks_dir = ws_config.notebooks_dir
+        config.workspace_config_dir = ws_config.workspace_config_dir
+
+        logger.debug(f"Patching {config.__class__.__name__}")
+        if isinstance(config, DockerConfig):
+            from phi.docker.docker_operator import patch_docker_config
+
+            # Pass down docker_env from WorkspaceConfig
+            config.docker_env = ws_config.docker_env
+            patch_docker_config(
+                config=config,
+                name_filter=target_name,
+                type_filter=target_type,
+                dry_run=dry_run,
+            )
+            num_configs_processed += 1
+        if isinstance(config, K8sConfig):
+            from phi.k8s.k8s_operator import patch_k8s_config
+
+            # Pass down k8s_env from WorkspaceConfig
+            config.k8s_env = ws_config.k8s_env
+            patch_k8s_config(
+                config=config,
+                name_filter=target_name,
+                type_filter=target_type,
+                dry_run=dry_run,
+            )
+            num_configs_processed += 1
+        if isinstance(config, AwsConfig):
+            from phi.aws.aws_operator import patch_aws_config
+
+            # Pass down common aws configuration from WorkspaceConfig
+            config.aws_region = ws_config.aws_region
+            config.aws_profile = ws_config.aws_profile
+            config.aws_config_file = ws_config.aws_config_file
+            config.aws_shared_credentials_file = ws_config.aws_shared_credentials_file
+            patch_aws_config(
+                config=config,
+                name_filter=target_name,
+                type_filter=target_type,
+                dry_run=dry_run,
+            )
+            num_configs_processed += 1
+
+    print_info(
+        f"\n# Configs processed: {num_configs_processed}/{num_configs_to_process}"
+    )
+    if num_configs_to_process == num_configs_processed:
+        if not dry_run:
+            print_subheading("Workspace patch success")
+    else:
+        print_subheading("Workspace patch failed")
+
+
 def init_workspace_data(ws_schema: WorkspaceSchema, phi_conf: PhiConf):
     """Initializes workspace data like cloud projects"""
 
